@@ -15,10 +15,9 @@
 #define LOGI(...) SDL_LogInfo(SDL_LOG_CATEGORY_APPLICATION, __VA_ARGS__)
 #define LOGW(...) SDL_LogWarn(SDL_LOG_CATEGORY_APPLICATION, __VA_ARGS__)
 #define LOGE(...) SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, __VA_ARGS__)
-#define LOGC(...) SDL_LogCritical(SDL_LOG_CATEGORY_APPLICATION, __VA_ARGS__)
 
 #define LOG_OOM() \
-    LOGC("OOM: %s:%d %s()", __FILE__, __LINE__, __func__)
+    LOGE("OOM: %s:%d %s()", __FILE__, __LINE__, __func__)
 
 void
 sc_set_log_level(enum sc_log_level level);
@@ -26,4 +25,14 @@ sc_set_log_level(enum sc_log_level level);
 enum sc_log_level
 sc_get_log_level(void);
 
+void
+sc_log(enum sc_log_level level, const char *fmt, ...);
+#define LOG(LEVEL, ...) sc_log((LEVEL), __VA_ARGS__)
+
+#ifdef _WIN32
+// Log system error (typically returned by GetLastError() or similar)
+bool
+sc_log_windows_error(const char *prefix, int error);
+#endif
+
 #endif
